@@ -28,6 +28,16 @@ CONDITIONS = {
 }    
 
 
+class KeyField(object):
+    index = False
+    
+    def __get__(self, instance, owner):
+        return instance.key
+
+    def __set__(self, instance, value):
+        instance.key = value if isinstance(value, unicode) else value.decode('utf-8')
+
+
 class BaseField(object):
     INDEX = {}
     CONDITION_TYPE = 'str'
@@ -38,15 +48,22 @@ class BaseField(object):
         self.index = index
 
     def __get__(self, instance, owner):
-        if self.key in instance._data:
-            return self.loads(instance._data[self.key])
+        if self.key in instance._cached_data:
+            return instance._cached_data[self.key]
+        if self.key in instance._raw_data:
+            ret = self.loads(instance._raw_data[self.key])
+            instance._cached_data[self.key] = ret
+            return ret
         return self.default
 
     def __set__(self, instance, value):
-        instance._data[self.key] = self.dumps(value)
+        instance._cached_data[self.key] = value
+        instance._raw_data[self.key] = self.dumps(value)
         
     def __delete__(self, instance):
-        del instance._data[self.key]
+        del instance._raw_data[self.key]
+        if self.key in instance._cached_data:
+            del instance._cached_data[self.key]
         
     def get_condition(self, abbr):
         return CONDITIONS.get(self.CONDITION_TYPE, {})[abbr]
@@ -226,12 +243,12 @@ def _compressed_factory(cls):
             super(self.__class__, self).__init__(*args, **kwargs)
             
         def dumps(self, value):
-            value = self.compressor.compress(value, self.compress_level)
-            return super(self.__class__, self).dumps(value)
+            value = super(self.__class__, self).dumps(value)
+            return self.compressor.compress(value, self.compress_level)
             
         def loads(self, s):
             s = self.compressor.decompress(s)
-            return super(self.__class__, self).loads(value)
+            return super(self.__class__, self).loads(s)
             
     Ret.__name__ = 'Compressed' + cls.__name__
     return Ret
@@ -241,16 +258,6 @@ CompressedUnicodeField = _compressed_factory(UnicodeField)
 CompressedDumpField = _compressed_factory(DumpField)
 
 
-class KeyField(object):
-    index = False
-    
-    def __get__(self, instance, owner):
-        return instance.key
-
-    def __set__(self, instance, value):
-        instance.key = value if isinstance(value, unicode) else value.decode('utf-8')
-
-
 if __name__ == "__main__":
     import doctest
     doctest.testmod()
\ No newline at end of file
